#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 25 13:40:50 2022

@author: peter.dieterich@tu-dresden.de

    usage: python3 -u A_CovSimData.py

"""


# import covariances
from myCov import covMatTemperedFBM as covTFBM 

import numpy as np
import scipy

import pandas as pd

#
# 1. define model and simulation parameters
#
H_sim    = 0.75     # Hurst coefficient [1]
DH_sim   = 50.0     # generalized diffusion coefficient [µm^2/min^(2 H)]
tauS_sim = 10.0     # tempering parameter [min]
mu_sim   = 0.0      # power-law tempering parameter [1]
vd_sim   = 8.0      # drift velocity [µm/min]

dt_sim    = 1.0      # simulation time step [min]
Lmax_sim  = 200      # maximal number of time steps in simulation
Mpath_sim = 100      # number of simulated paths


# 
# 2. calculate the covariance and the corresponding Cholesky decompostion
#
Ccov = covTFBM(dt=dt_sim, DH=DH_sim, H=H_sim, L=Lmax_sim, tauS=tauS_sim, mu=mu_sim, tag="powerlaw")
Lcov = np.linalg.cholesky(Ccov)


# 
# 3. simulate Mpath_sim paths of lenght Lmax_sim
#
xxx = np.zeros([Mpath_sim, Lmax_sim]) 
ttt = np.linspace(start=1, stop=Lmax_sim+1, num=Lmax_sim)*dt_sim
for k in range(Mpath_sim):
    xi = scipy.random.normal(loc=0.0, scale=1.0, size=Lmax_sim)
    xx = np.dot(Lcov, xi) 
    tt = np.zeros(len(xx))
    for i in range(Lmax_sim):
        xx[i] = xx[i]+(i+1)*dt_sim*vd_sim
    xxx[k] = xx
   

# 
# 4. write simulated paths to excel file
#
writer    = pd.ExcelWriter('simulatedPaths_data.xlsx')
workbook  = writer.book
format1   = workbook.add_format({'num_format': '0.0000'})
df = pd.DataFrame()
xna = 'time'; df[xna] = ttt.tolist()
for k in range(Mpath_sim):
    xna = 'x'+str(1000+k)[1:]; df[xna] = xxx[k].tolist()

df.to_excel(writer, sheet_name="paths", startrow=0, startcol=0, header=True, index=True)    
writer.save()